@@ -295,11 +295,24 @@ def generate_typed_stubs(modpath):
                 # gen.add_import_line('from typing import {}\n'.format('TypeVar'))
                 gen._output = ['{} = TypeVar("{}")\n'.format(type_var_name, type_var_name)] + gen._output
 
+            # Check for a special user header variable we pull in verbatim
+            import mypy
+            user_header = None
+            for d in mod.ast.defs:
+                if isinstance(d, mypy.nodes.AssignmentStmt):
+                    try:
+                        if len(d.lvalues) == 1 and d.lvalues[0].name == '__docstubs__':
+                            user_header = d.rvalue.value
+                    except AttributeError:
+                        ...
+
+            if user_header is not None:
+                gen._output = [user_header] + gen._output
+
             # Hack for specific module
-            # if mod.path.endswith('util_path.py'):
-            #     gen.add_typing_import('TypeVar')
-            #     # hack for variable inheritence
-            #     gen._output = ['import pathlib\nimport os\n', "_PathBase = pathlib.WindowsPath if os.name == 'nt' else pathlib.PosixPath\n"] + gen._output
+            # if mod.path.endswith('coco_objects1d.py'):
+            #     import xdev
+            #     xdev.embed()
 
             text = ''.join(gen.output())
             text = postprocess_hacks(text, mod)
@@ -532,6 +545,12 @@ def hacked_typing_info(type_name):
         # into the type if given in the docstring
         type_name = type_name.replace('callable', 'Callable')
 
+    import re
+    type_name = re.sub(r'\bor\b', '|', type_name)
+
+    if type_name == '?':
+        type_name = 'Any'
+
     if '|' in type_name:
         add_typing_import('Union')
         add_import_line('from typing import {}\n'.format('Union'))
@@ -661,6 +680,7 @@ class ExtendedStubGenerator(StubGenerator):
             print('o.name = {!r}'.format(o.name))
 
         import ubelt as ub
+        # Parse extra information out of the docstring
         name_to_parsed_docstr_info = {}
         return_parsed_docstr_info = None
         fullname = o.name
@@ -734,13 +754,18 @@ class ExtendedStubGenerator(StubGenerator):
             self_inits_lut = dict(self_inits)
             # The docstring is the single source of truth, respect it.
             pseudo_inits = []
-            for name, info in self._docstring_class_attr_infos.items():
+            if self._docstring_class_attr_infos is None:
+                _docstring_class_attr_infos = {}
+            else:
+                _docstring_class_attr_infos = self._docstring_class_attr_infos
+
+            for name, info in _docstring_class_attr_infos.items():
                 if name in self_inits_lut:
                     pseudo_inits.append((name, self_inits_lut[name]))
                 else:
                     pseudo_inits.append((name, None))
             # Maybe we shouldnt do this if there is an Attributes section?
-            pseudo_inits.extend(list(ub.dict_diff(self_inits_lut, self._docstring_class_attr_infos).items()))
+            pseudo_inits.extend(list(ub.dict_diff(self_inits_lut, _docstring_class_attr_infos).items()))
 
             for init, value in pseudo_inits:
                 if init in self.method_names:
@@ -750,8 +775,8 @@ class ExtendedStubGenerator(StubGenerator):
                 # Use the init docstring to get a hint for the type
                 annotation = None
                 # The class attributes should override the init signature
-                if init in self._docstring_class_attr_infos:
-                    typename = self._docstring_class_attr_infos[init]['type']
+                if init in _docstring_class_attr_infos:
+                    typename = _docstring_class_attr_infos[init]['type']
                     try:
                         annotation = fastparse.parse_type_string(typename, 'Any', 0, 0)
                     except Exception:
@@ -775,13 +800,42 @@ class ExtendedStubGenerator(StubGenerator):
         self.clear_decorators()
         self.add("%s%sdef %s(" % (self._indent, 'async ' if o.is_coroutine else '', o.name))
         self.record_name(o.name)
-        # import ubelt as ub
-        # if o.name == 'dzip':
-        #     import xdev
-        #     xdev.embed()
 
-        # print('o = {!r}'.format(o))
-        # print('o.arguments = {!r}'.format(o.arguments))
+        DEVELOPER_DEBUGGING = 0
+        if DEVELOPER_DEBUGGING:
+            # Set this to the name function we are going to debug
+            function_to_debug = 'show_chipmatch2'
+            print(f'o.name={o.name}')
+            if o.name == function_to_debug:
+                print('o = {!r}'.format(o))
+                print('o.arguments = {!r}'.format(o.arguments))
+                import xdev
+                xdev.embed()
+
+        # ------------------------------------------
+        # Enrich doctypes with inferable information
+        # ------------------------------------------
+        # Do a quick initial pass to check to compare the parsed docstr types
+        # to default values if they exist. If the default value is something
+        # like None, but the existing type annotation isn't marked as optional
+        # we can insert that for the user.
+        name_to_argument = {arg_.variable.name: arg_ for arg_ in o.arguments}
+        check_names = set(name_to_argument) & set(name_to_parsed_docstr_info)
+        for name in check_names:
+            arg_ = name_to_argument[name]
+            if arg_.initializer is not None:
+                # TODO: find a better way of checking if the default value
+                # matches the type of the given doctype and extend the
+                # doctype if needbe. For now we are hacking it to
+                # handle None specificaly.
+                if hasattr(arg_.initializer, 'name') and arg_.initializer.name == 'None':
+                    info = name_to_parsed_docstr_info[name]
+                    doctype_str = info['type'].replace(' ', '')
+                    if all(n not in doctype_str for n in {'None', 'Optional'}):
+                        info['type'] = info['type'] + ' | None'
+                        self.add_typing_import('Union')
+        # ------------------------------------------
+
         args: List[str] = []
         for i, arg_ in enumerate(o.arguments):
             var = arg_.variable
@@ -800,10 +854,6 @@ class ExtendedStubGenerator(StubGenerator):
                         # import mypy.types as mypy_types
                         # globals_ = {**mypy_types.__dict__}
                         try:
-                            # # got = mypy_types.deserialize_type(doc_type_str)
-                            # got = eval(doc_type_str, globals_)
-                            # got = mypy_types.get_proper_type(got)
-                            # got = mypy_types.Iterable
                             got = fastparse.parse_type_string(doc_type_str, 'Any', 0, 0)
                         except Exception as ex:
                             print('ex = {!r}'.format(ex))
@@ -1005,5 +1055,5 @@ if __name__ == '__main__':
     CommandLine:
         python -m xdev.cli.gen_typed_stubs
     """
-    from xdev.cli.main import DocstrStubgenCLI
+    from xdev.cli.docstr_stubgen import DocstrStubgenCLI
     DocstrStubgenCLI.main()
